@@ -249,7 +249,7 @@ class W_BZ2File(W_File):
             space = self.space
             raise oefmt(space.w_ValueError, "invalid mode: '%s'", mode)
 
-    @unwrap_spec(mode=str, buffering=int, compresslevel=int)
+    @unwrap_spec(mode='text', buffering=int, compresslevel=int)
     def direct_bz2__init__(self, w_name, mode='r', buffering=-1,
                            compresslevel=9):
         self.direct_close()
@@ -265,7 +265,8 @@ class W_BZ2File(W_File):
 
     _exposed_method_names = []
     W_File._decl.im_func(locals(), "bz2__init__",
-                         """Opens a BZ2-compressed file.""")
+                         """Opens a BZ2-compressed file.""",
+                         wrapresult="space.w_None")
     # XXX ^^^ hacking hacking... can't just use the name "__init__" again
     # because the RTyper is confused about the two direct__init__() with
     # a different signature, confusion caused by the fact that
@@ -277,9 +278,6 @@ class W_BZ2File(W_File):
             head = "closed"
         else:
             head = "open"
-        w_name = self.w_name
-        if w_name is None:
-            w_name = self.space.wrap('?')
         info = "%s bz2.BZ2File %s, mode '%s'" % (head, self.getdisplayname(),
                                                  self.mode)
         return self.getrepr(self.space, info)
@@ -287,13 +285,13 @@ class W_BZ2File(W_File):
 def descr_bz2file__new__(space, w_subtype, __args__):
     bz2file = space.allocate_instance(W_BZ2File, w_subtype)
     W_BZ2File.__init__(bz2file, space)
-    return space.wrap(bz2file)
+    return bz2file
 
 same_attributes_as_in_file = list(W_File._exposed_method_names)
 same_attributes_as_in_file.remove('__init__')
 same_attributes_as_in_file.extend([
     'name', 'mode', 'encoding', 'closed', 'newlines', 'softspace',
-    'writelines', '__exit__', '__weakref__'])
+    'writelines', '__exit__', '__weakref__', 'write'])
 
 W_BZ2File.typedef = TypeDef(
     "BZ2File",
@@ -318,6 +316,7 @@ newlines are available only when reading.""",
     **dict([(name, W_File.typedef.rawdict[name])
             for name in same_attributes_as_in_file]))
 
+
 # ____________________________________________________________
 
 def open_bz2file_as_stream(space, w_path, mode="r", buffering=-1,
@@ -329,7 +328,7 @@ def open_bz2file_as_stream(space, w_path, mode="r", buffering=-1,
         raise oefmt(space.w_ValueError, "cannot open in read-write mode")
     if basemode == "a":
         raise oefmt(space.w_ValueError, "cannot append to bz2 file")
-    stream = open_path_helper(space.str0_w(w_path), os_flags, False)
+    stream = open_path_helper(space.fsencode_w(w_path), os_flags, False)
     if reading:
         bz2stream = ReadBZ2Filter(space, stream, buffering)
         buffering = 0     # by construction, the ReadBZ2Filter acts like
@@ -414,7 +413,7 @@ class ReadBZ2Filter(Stream):
                 raise oefmt(self.space.w_EOFError,
                             "compressed file ended before the logical "
                             "end-of-the-stream was detected")
-            result = self.space.str_w(w_result)
+            result = self.space.bytes_w(w_result)
             self.readlength += len(result)
         else:
             result = ""
@@ -444,7 +443,7 @@ class ReadBZ2Filter(Stream):
                     self.finished = True
                     return ""
                 raise
-            self.buffer = self.space.str_w(w_read)
+            self.buffer = self.space.bytes_w(w_read)
             self.pos = 0
         if len(self.buffer) - self.pos >= n:
             pos = self.pos
@@ -479,11 +478,11 @@ class WriteBZ2Filter(Stream):
         self.writtenlength = 0
 
     def close1(self, closefileno):
-        self.stream.write(self.space.str_w(self.compressor.flush()))
+        self.stream.write(self.space.bytes_w(self.compressor.flush()))
         self.stream.close1(closefileno)
 
     def write(self, data):
-        self.stream.write(self.space.str_w(self.compressor.compress(data)))
+        self.stream.write(self.space.bytes_w(self.compressor.compress(data)))
         self.writtenlength += len(data)
 
     def tell(self):
@@ -504,9 +503,8 @@ class WriteBZ2Filter(Stream):
 @unwrap_spec(compresslevel=int)
 def descr_compressor__new__(space, w_subtype, compresslevel=9):
     x = space.allocate_instance(W_BZ2Compressor, w_subtype)
-    x = space.interp_w(W_BZ2Compressor, x)
     W_BZ2Compressor.__init__(x, space, compresslevel)
-    return space.wrap(x)
+    return x
 
 class W_BZ2Compressor(W_Root):
     """BZ2Compressor([compresslevel=9]) -> compressor object
@@ -614,9 +612,8 @@ W_BZ2Compressor.typedef = TypeDef("BZ2Compressor",
 
 def descr_decompressor__new__(space, w_subtype):
     x = space.allocate_instance(W_BZ2Decompressor, w_subtype)
-    x = space.interp_w(W_BZ2Decompressor, x)
     W_BZ2Decompressor.__init__(x, space)
-    return space.wrap(x)
+    return x
 
 
 class W_BZ2Decompressor(W_Root):
@@ -704,7 +701,8 @@ class W_BZ2Decompressor(W_Root):
 W_BZ2Decompressor.typedef = TypeDef("BZ2Decompressor",
     __doc__ = W_BZ2Decompressor.__doc__,
     __new__ = interp2app(descr_decompressor__new__),
-    unused_data = interp_attrproperty("unused_data", W_BZ2Decompressor),
+    unused_data = interp_attrproperty("unused_data", W_BZ2Decompressor,
+        wrapfn="newbytes"),
     decompress = interp2app(W_BZ2Decompressor.decompress),
 )
 
@@ -750,7 +748,7 @@ def compress(space, data, compresslevel=9):
 
                 res = out.make_result_string()
                 BZ2_bzCompressEnd(bzs)
-                return space.wrap(res)
+                return space.newbytes(res)
 
 @unwrap_spec(data='bufferstr')
 def decompress(space, data):
@@ -761,7 +759,7 @@ def decompress(space, data):
 
     in_bufsize = len(data)
     if in_bufsize == 0:
-        return space.wrap("")
+        return space.newbytes("")
 
     with lltype.scoped_alloc(bz_stream.TO, zero=True) as bzs:
         with rffi.scoped_nonmovingbuffer(data) as in_buf:
@@ -790,4 +788,4 @@ def decompress(space, data):
 
                 res = out.make_result_string()
                 BZ2_bzDecompressEnd(bzs)
-                return space.wrap(res)
+                return space.newbytes(res)
