@@ -2,25 +2,28 @@ import pypy.module.cppyy.capi as capi
 
 from pypy.interpreter.error import OperationError, oefmt
 from pypy.interpreter.gateway import interp2app, unwrap_spec
-from pypy.interpreter.typedef import TypeDef, GetSetProperty, interp_attrproperty
+from pypy.interpreter.typedef import TypeDef, GetSetProperty, interp_attrproperty, interp_attrproperty_w
 from pypy.interpreter.baseobjspace import W_Root
 
 from rpython.rtyper.lltypesystem import rffi, lltype, llmemory
 
-from rpython.rlib import jit, rdynload, rweakref
+from rpython.rlib import jit, rdynload, rweakref, rgc
 from rpython.rlib import jit_libffi, clibffi
+from rpython.rlib.objectmodel import we_are_translated, keepalive_until_here
 
-from pypy.module.cppyy import converter, executor, helper
+from pypy.module._cffi_backend import ctypefunc
+from pypy.module.cppyy import converter, executor, ffitypes, helper
 
 
 class FastCallNotPossible(Exception):
     pass
 
 # overload priorities: lower is preferred
-priority = { 'void*'  : 100,
-             'void**' : 100,
-             'float'  :  30,
-             'double' :  10, }
+priority = { 'void*'         : 100,
+             'void**'        : 100,
+             'float'         :  30,
+             'double'        :  10,
+             'const string&' :   1, } # solves a specific string ctor overload
 
 from rpython.rlib.listsort import make_timsort_class
 CPPMethodBaseTimSort = make_timsort_class()
@@ -28,12 +31,19 @@ class CPPMethodSort(CPPMethodBaseTimSort):
     def lt(self, a, b):
         return a.priority() < b.priority()
 
-@unwrap_spec(name=str)
+@unwrap_spec(name='text')
 def load_dictionary(space, name):
     try:
         cdll = capi.c_load_dictionary(name)
+        if not cdll:
+           raise OperationError(space.w_RuntimeError, space.newtext(str("could not load dictionary " + name)))
+
     except rdynload.DLOpenError as e:
-        raise OperationError(space.w_RuntimeError, space.wrap(str(e.msg)))
+        if hasattr(space, "fake"):      # FakeSpace fails e.msg (?!)
+            errmsg = "failed to load cdll"
+        else:
+            errmsg = e.msg
+        raise OperationError(space.w_RuntimeError, space.newtext(str(errmsg)))
     return W_CPPLibrary(space, cdll)
 
 class State(object):
@@ -53,21 +63,21 @@ def get_nullptr(space):
     if state.w_nullptr is None:
         from pypy.module._rawffi.interp_rawffi import unpack_simple_shape
         from pypy.module._rawffi.array import W_Array, W_ArrayInstance
-        arr = space.interp_w(W_Array, unpack_simple_shape(space, space.wrap('P')))
+        arr = space.interp_w(W_Array, unpack_simple_shape(space, space.newtext('P')))
         # TODO: fix this hack; fromaddress() will allocate memory if address
         # is null and there seems to be no way around it (ll_buffer can not
         # be touched directly)
         nullarr = arr.fromaddress(space, rffi.cast(rffi.ULONG, 0), 0)
         assert isinstance(nullarr, W_ArrayInstance)
         nullarr.free(space)
-        state.w_nullptr = space.wrap(nullarr)
+        state.w_nullptr = nullarr
     return state.w_nullptr
 
-@unwrap_spec(name=str)
+@unwrap_spec(name='text')
 def resolve_name(space, name):
-    return space.wrap(capi.c_resolve_name(space, name))
+    return space.newtext(capi.c_resolve_name(space, name))
 
-@unwrap_spec(name=str)
+@unwrap_spec(name='text')
 def scope_byname(space, name):
     true_name = capi.c_resolve_name(space, name)
 
@@ -95,7 +105,7 @@ def scope_byname(space, name):
 
     return None
 
-@unwrap_spec(name=str)
+@unwrap_spec(name='text')
 def template_byname(space, name):
     state = space.fromcache(State)
     try:
@@ -103,17 +113,15 @@ def template_byname(space, name):
     except KeyError:
         pass
 
-    opaque_handle = capi.c_get_template(space, name)
-    assert lltype.typeOf(opaque_handle) == capi.C_TYPE
-    if opaque_handle:
-        cpptemplate = W_CPPTemplateType(space, name, opaque_handle)
+    if capi.c_is_template(space, name):
+        cpptemplate = W_CPPTemplateType(space, name)
         state.cpptemplate_cache[name] = cpptemplate
         return cpptemplate
 
     return None
 
 def std_string_name(space):
-    return space.wrap(capi.std_string_name)
+    return space.newtext(capi.std_string_name)
 
 @unwrap_spec(w_callback=W_Root)
 def set_class_generator(space, w_callback):
@@ -126,7 +134,7 @@ def set_function_generator(space, w_callback):
     state.w_fngen_callback = w_callback
 
 def register_class(space, w_pycppclass):
-    w_cppclass = space.findattr(w_pycppclass, space.wrap("_cpp_proxy"))
+    w_cppclass = space.findattr(w_pycppclass, space.newtext("_cpp_proxy"))
     cppclass = space.interp_w(W_CPPClass, w_cppclass, can_be_None=False)
     # add back-end specific method pythonizations (doing this on the wrapped
     # class allows simple aliasing of methods)
@@ -171,7 +179,7 @@ class CPPMethod(object):
         self.converters = None
         self.executor = None
         self.cif_descr = lltype.nullptr(jit_libffi.CIF_DESCRIPTION)
-        self._funcaddr = lltype.nullptr(rffi.VOIDP.TO)
+        self._funcaddr = lltype.nullptr(capi.C_FUNC_PTR.TO)
         self.uses_local = False
 
     @staticmethod
@@ -189,6 +197,8 @@ class CPPMethod(object):
 
     @jit.unroll_safe
     def call(self, cppthis, args_w):
+        jit.promote(self)
+
         assert lltype.typeOf(cppthis) == capi.C_OBJECT
 
         # check number of given arguments against required (== total - defaults)
@@ -201,7 +211,7 @@ class CPPMethod(object):
         if self.converters is None:
             try:
                 self._setup(cppthis)
-            except Exception as e:
+            except Exception:
                 pass
 
         # some calls, e.g. for ptr-ptr or reference need a local array to store data for
@@ -253,12 +263,56 @@ class CPPMethod(object):
                 data = capi.exchange_address(buffer, cif_descr, j+1)
                 conv.default_argument_libffi(self.space, data)
 
+            assert self._funcaddr
             w_res = self.executor.execute_libffi(
                 self.space, cif_descr, self._funcaddr, buffer)
         finally:
             lltype.free(buffer, flavor='raw')
+            keepalive_until_here(args_w)
         return w_res
 
+    # from ctypefunc; have my own version for annotater purposes and to disable
+    # memory tracking (method live time is longer than the tests)
+    @jit.dont_look_inside
+    def _rawallocate(self, builder):
+        builder.space = self.space
+
+        # compute the total size needed in the CIF_DESCRIPTION buffer
+        builder.nb_bytes = 0
+        builder.bufferp = lltype.nullptr(rffi.CCHARP.TO)
+        builder.fb_build()
+
+        # allocate the buffer
+        if we_are_translated():
+            rawmem = lltype.malloc(rffi.CCHARP.TO, builder.nb_bytes,
+                                   flavor='raw', track_allocation=False)
+            rawmem = rffi.cast(jit_libffi.CIF_DESCRIPTION_P, rawmem)
+        else:
+            # gross overestimation of the length below, but too bad
+            rawmem = lltype.malloc(jit_libffi.CIF_DESCRIPTION_P.TO, builder.nb_bytes,
+                                   flavor='raw', track_allocation=False)
+
+        # the buffer is automatically managed from the W_CTypeFunc instance
+        self.cif_descr = rawmem
+
+        # call again fb_build() to really build the libffi data structures
+        builder.bufferp = rffi.cast(rffi.CCHARP, rawmem)
+        builder.fb_build()
+        assert builder.bufferp == rffi.ptradd(rffi.cast(rffi.CCHARP, rawmem),
+                                              builder.nb_bytes)
+
+        # fill in the 'exchange_*' fields
+        builder.fb_build_exchange(rawmem)
+
+        # fill in the extra fields
+        builder.fb_extra_fields(rawmem)
+
+        # call libffi's ffi_prep_cif() function
+        res = jit_libffi.jit_ffi_prep_cif(rawmem)
+        if res != clibffi.FFI_OK:
+            raise oefmt(self.space.w_SystemError,
+                        "libffi failed to build this function type")
+
     def _setup(self, cppthis):
         self.converters = [converter.get_converter(self.space, arg_type, arg_dflt)
                                for arg_type, arg_dflt in self.arg_defs]
@@ -273,78 +327,36 @@ class CPPMethod(object):
         # Each CPPMethod corresponds one-to-one to a C++ equivalent and cppthis
         # has been offset to the matching class. Hence, the libffi pointer is
         # uniquely defined and needs to be setup only once.
-        methgetter = capi.c_get_methptr_getter(self.space, self.scope, self.index)
-        if methgetter and cppthis:      # methods only for now
-            cif_descr = lltype.nullptr(jit_libffi.CIF_DESCRIPTION)
+        funcaddr = capi.c_get_function_address(self.space, self.scope, self.index)
+        if funcaddr and cppthis:      # methods only for now
+            state = self.space.fromcache(ffitypes.State)
+
+            # argument type specification (incl. cppthis)
+            fargs = []
             try:
-                funcaddr = methgetter(rffi.cast(capi.C_OBJECT, cppthis))
-                self._funcaddr = rffi.cast(rffi.VOIDP, funcaddr)
-
-                nargs = len(self.arg_defs) + 1                   # +1: cppthis
-
-                # memory block for CIF description (note: not tracked as the life
-                # time of methods is normally the duration of the application)
-                size = llmemory.sizeof(jit_libffi.CIF_DESCRIPTION, nargs)
-
-                # allocate the buffer
-                cif_descr = lltype.malloc(jit_libffi.CIF_DESCRIPTION_P.TO,
-                                          llmemory.raw_malloc_usage(size),
-                                          flavor='raw', track_allocation=False)
-
-                # array of 'ffi_type*' values, one per argument
-                size = rffi.sizeof(jit_libffi.FFI_TYPE_P) * nargs
-                atypes = lltype.malloc(rffi.CCHARP.TO, llmemory.raw_malloc_usage(size),
-                                       flavor='raw', track_allocation=False)
-                cif_descr.atypes = rffi.cast(jit_libffi.FFI_TYPE_PP, atypes)
-
-                # argument type specification
-                cif_descr.atypes[0] = jit_libffi.types.pointer   # cppthis
+                fargs.append(state.c_voidp)
                 for i, conv in enumerate(self.converters):
-                    if not conv.libffitype:
-                        raise FastCallNotPossible
-                    cif_descr.atypes[i+1] = conv.libffitype
+                    fargs.append(conv.cffi_type(self.space))
+                fresult = self.executor.cffi_type(self.space)
+            except:
+                raise FastCallNotPossible
 
-                # result type specification
-                cif_descr.rtype = self.executor.libffitype
+            # the following is derived from _cffi_backend.ctypefunc
+            builder = ctypefunc.CifDescrBuilder(fargs[:], fresult, clibffi.FFI_DEFAULT_ABI)
+            try:
+                self._rawallocate(builder)
+            except OperationError as e:
+                if not e.match(self.space, self.space.w_NotImplementedError):
+                    raise
+                # else, eat the NotImplementedError.  We will get the
+                # exception if we see an actual call
+                if self.cif_descr:   # should not be True, but you never know
+                    lltype.free(self.cif_descr, flavor='raw')
+                    self.cif_descr = lltype.nullptr(jit_libffi.CIF_DESCRIPTION)
+                raise FastCallNotPossible
 
-                # exchange ---
-
-                # first, enough room for an array of 'nargs' pointers
-                exchange_offset = rffi.sizeof(rffi.CCHARP) * nargs
-                exchange_offset = (exchange_offset + 7) & ~7     # alignment
-                cif_descr.exchange_result = exchange_offset
-
-                # then enough room for the result, rounded up to sizeof(ffi_arg)
-                exchange_offset += max(rffi.getintfield(cif_descr.rtype, 'c_size'),
-                                       jit_libffi.SIZE_OF_FFI_ARG)
-
-                # loop over args
-                for i in range(nargs):
-                    exchange_offset = (exchange_offset + 7) & ~7 # alignment
-                    cif_descr.exchange_args[i] = exchange_offset
-                    exchange_offset += rffi.getintfield(cif_descr.atypes[i], 'c_size')
-
-                # store the exchange data size
-                cif_descr.exchange_size = exchange_offset
-
-                # --- exchange
-
-                # extra
-                cif_descr.abi = clibffi.FFI_DEFAULT_ABI
-                cif_descr.nargs = len(self.arg_defs) + 1         # +1: cppthis
-
-                res = jit_libffi.jit_ffi_prep_cif(cif_descr)
-                if res != clibffi.FFI_OK:
-                    raise FastCallNotPossible
-
-            except Exception as e:
-                if cif_descr:
-                    lltype.free(cif_descr.atypes, flavor='raw', track_allocation=False)
-                    lltype.free(cif_descr, flavor='raw', track_allocation=False)
-                cif_descr = lltype.nullptr(jit_libffi.CIF_DESCRIPTION)
-                self._funcaddr = lltype.nullptr(rffi.VOIDP.TO)
-
-            self.cif_descr = cif_descr
+            # success ...
+            self._funcaddr = funcaddr
 
     @jit.unroll_safe
     def prepare_arguments(self, args_w, call_local):
@@ -388,9 +400,9 @@ class CPPMethod(object):
             total_arg_priority += p
         return total_arg_priority
 
+    @rgc.must_be_light_finalizer
     def __del__(self):
         if self.cif_descr:
-            lltype.free(self.cif_descr.atypes, flavor='raw')
             lltype.free(self.cif_descr, flavor='raw')
 
     def __repr__(self):
@@ -429,9 +441,9 @@ class CPPTemplatedCall(CPPMethod):
         assert lltype.typeOf(cppthis) == capi.C_OBJECT
         for i in range(len(args_w)):
             try:
-                s = self.space.str_w(args_w[i])
+                s = self.space.text_w(args_w[i])
             except OperationError:
-                s = self.space.str_w(self.space.getattr(args_w[i], self.space.wrap('__name__')))
+                s = self.space.text_w(self.space.getattr(args_w[i], self.space.newtext('__name__')))
             s = capi.c_resolve_name(self.space, s)
             if s != self.templ_args[i]:
                 raise oefmt(self.space.w_TypeError,
@@ -537,7 +549,9 @@ class W_CPPOverload(W_Root):
         # only get here if all overloads failed ...
         errmsg = 'none of the %d overloaded methods succeeded. Full details:' % len(self.functions)
         if hasattr(self.space, "fake"):     # FakeSpace fails errorstr (see below)
-            raise OperationError(self.space.w_TypeError, self.space.wrap(errmsg))
+            raise OperationError(self.space.w_TypeError, self.space.newtext(errmsg))
+        w_exc_type = None
+        all_same_type = True
         for i in range(len(self.functions)):
             cppyyfunc = self.functions[i]
             try:
@@ -546,6 +560,10 @@ class W_CPPOverload(W_Root):
                 # special case if there's just one function, to prevent clogging the error message
                 if len(self.functions) == 1:
                     raise
+                if w_exc_type is None:
+                    w_exc_type = e.w_type
+                elif all_same_type and not e.match(self.space, w_exc_type):
+                    all_same_type = False
                 errmsg += '\n  '+cppyyfunc.signature()+' =>\n'
                 errmsg += '    '+e.errorstr(self.space)
             except Exception as e:
@@ -554,13 +572,16 @@ class W_CPPOverload(W_Root):
                 errmsg += '\n  '+cppyyfunc.signature()+' =>\n'
                 errmsg += '    Exception: '+str(e)
 
-        raise OperationError(self.space.w_TypeError, self.space.wrap(errmsg))
+        if all_same_type and w_exc_type is not None:
+            raise OperationError(w_exc_type, self.space.newtext(errmsg))
+        else:
+            raise OperationError(self.space.w_TypeError, self.space.newtext(errmsg))
 
     def signature(self):
         sig = self.functions[0].signature()
         for i in range(1, len(self.functions)):
             sig += '\n'+self.functions[i].signature()
-        return self.space.wrap(sig)
+        return self.space.newtext(sig)
 
     def __repr__(self):
         return "W_CPPOverload(%s)" % [f.signature() for f in self.functions]
@@ -632,8 +653,8 @@ class W_CPPDataMember(W_Root):
         self.converter = converter.get_converter(self.space, type_name, '')
         self.offset = offset
 
-    def get_returntype(self):
-        return self.space.wrap(self.converter.name)
+    def is_static(self):
+        return self.space.w_False
 
     def _get_offset(self, cppinstance):
         if cppinstance:
@@ -662,13 +683,16 @@ class W_CPPDataMember(W_Root):
 
 W_CPPDataMember.typedef = TypeDef(
     'CPPDataMember',
-    get_returntype = interp2app(W_CPPDataMember.get_returntype),
+    is_static = interp2app(W_CPPDataMember.is_static),
     __get__ = interp2app(W_CPPDataMember.get),
     __set__ = interp2app(W_CPPDataMember.set),
 )
 W_CPPDataMember.typedef.acceptable_as_base_class = False
 
 class W_CPPStaticData(W_CPPDataMember):
+    def is_static(self):
+        return self.space.w_True
+
     @jit.elidable_promote()
     def _get_offset(self, cppinstance):
         return self.offset
@@ -682,7 +706,7 @@ class W_CPPStaticData(W_CPPDataMember):
 
 W_CPPStaticData.typedef = TypeDef(
     'CPPStaticData',
-    get_returntype = interp2app(W_CPPStaticData.get_returntype),
+    is_static = interp2app(W_CPPStaticData.is_static),
     __get__ = interp2app(W_CPPStaticData.get),
     __set__ = interp2app(W_CPPStaticData.set),
 )
@@ -750,7 +774,7 @@ class W_CPPScope(W_Root):
         return capi.c_scoped_final_name(self.space, self.handle)
 
     def get_method_names(self):
-        return self.space.newlist([self.space.wrap(name) for name in self.methods])
+        return self.space.newlist([self.space.newtext(name) for name in self.methods])
 
     def get_overload(self, name):
         try:
@@ -762,7 +786,7 @@ class W_CPPScope(W_Root):
         return new_method
 
     def get_datamember_names(self):
-        return self.space.newlist([self.space.wrap(name) for name in self.datamembers])
+        return self.space.newlist([self.space.newtext(name) for name in self.datamembers])
 
     def get_datamember(self, name):
         try:
@@ -812,21 +836,20 @@ class W_CPPNamespace(W_CPPScope):
             arg_defs.append((arg_type, arg_dflt))
         return CPPFunction(self.space, self, index, arg_defs, args_required)
 
+    def _build_methods(self):
+        pass       # force lazy lookups in namespaces
+
     def _make_datamember(self, dm_name, dm_idx):
         type_name = capi.c_datamember_type(self.space, self, dm_idx)
         offset = capi.c_datamember_offset(self.space, self, dm_idx)
+        if offset == -1:
+            raise self.missing_attribute_error(dm_name)
         datamember = W_CPPStaticData(self.space, self, type_name, offset)
         self.datamembers[dm_name] = datamember
         return datamember
 
     def _find_datamembers(self):
-        num_datamembers = capi.c_num_datamembers(self.space, self)
-        for i in range(num_datamembers):
-            if not capi.c_is_publicdata(self.space, self, i):
-                continue
-            datamember_name = capi.c_datamember_name(self.space, self, i)
-            if not datamember_name in self.datamembers:
-                self._make_datamember(datamember_name, i)
+        pass       # force lazy lookups in namespaces
 
     def find_overload(self, meth_name):
         indices = capi.c_method_indices_from_name(self.space, self, meth_name)
@@ -856,26 +879,26 @@ class W_CPPNamespace(W_CPPScope):
         alldir = []
         for i in range(capi.c_num_scopes(self.space, self)):
             sname = capi.c_scope_name(self.space, self, i)
-            if sname: alldir.append(self.space.wrap(sname))
+            if sname: alldir.append(self.space.newtext(sname))
         allmeth = {}
         for i in range(capi.c_num_methods(self.space, self)):
             idx = capi.c_method_index_at(self.space, self, i)
             mname = capi.c_method_name(self.space, self, idx)
             if mname: allmeth.setdefault(mname, 0)
         for m in allmeth.keys():
-            alldir.append(self.space.wrap(m))
+            alldir.append(self.space.newtext(m))
         for i in range(capi.c_num_datamembers(self.space, self)):
             dname = capi.c_datamember_name(self.space, self, i)
-            if dname: alldir.append(self.space.wrap(dname))
+            if dname: alldir.append(self.space.newtext(dname))
         return self.space.newlist(alldir)
         
 
 W_CPPNamespace.typedef = TypeDef(
     'CPPNamespace',
     get_method_names = interp2app(W_CPPNamespace.get_method_names),
-    get_overload = interp2app(W_CPPNamespace.get_overload, unwrap_spec=['self', str]),
+    get_overload = interp2app(W_CPPNamespace.get_overload, unwrap_spec=['self', 'text']),
     get_datamember_names = interp2app(W_CPPNamespace.get_datamember_names),
-    get_datamember = interp2app(W_CPPNamespace.get_datamember, unwrap_spec=['self', str]),
+    get_datamember = interp2app(W_CPPNamespace.get_datamember, unwrap_spec=['self', 'text']),
     is_namespace = interp2app(W_CPPNamespace.is_namespace),
     __dir__ = interp2app(W_CPPNamespace.ns__dir__),
 )
@@ -920,6 +943,8 @@ class W_CPPClass(W_CPPScope):
             datamember_name = capi.c_datamember_name(self.space, self, i)
             type_name = capi.c_datamember_type(self.space, self, i)
             offset = capi.c_datamember_offset(self.space, self, i)
+            if offset == -1:
+                continue      # dictionary problem; raises AttributeError on use
             is_static = bool(capi.c_is_staticdata(self.space, self, i))
             if is_static:
                 datamember = W_CPPStaticData(self.space, self, type_name, offset)
@@ -952,19 +977,19 @@ class W_CPPClass(W_CPPScope):
         num_bases = capi.c_num_bases(self.space, self)
         for i in range(num_bases):
             base_name = capi.c_base_name(self.space, self, i)
-            bases.append(self.space.wrap(base_name))
+            bases.append(self.space.newtext(base_name))
         return self.space.newlist(bases)
 
 W_CPPClass.typedef = TypeDef(
     'CPPClass',
-    type_name = interp_attrproperty('name', W_CPPClass),
+    type_name = interp_attrproperty('name', W_CPPClass, wrapfn="newtext"),
     get_base_names = interp2app(W_CPPClass.get_base_names),
     get_method_names = interp2app(W_CPPClass.get_method_names),
-    get_overload = interp2app(W_CPPClass.get_overload, unwrap_spec=['self', str]),
+    get_overload = interp2app(W_CPPClass.get_overload, unwrap_spec=['self', 'text']),
     get_datamember_names = interp2app(W_CPPClass.get_datamember_names),
-    get_datamember = interp2app(W_CPPClass.get_datamember, unwrap_spec=['self', str]),
+    get_datamember = interp2app(W_CPPClass.get_datamember, unwrap_spec=['self', 'text']),
     is_namespace = interp2app(W_CPPClass.is_namespace),
-    dispatch = interp2app(W_CPPClass.dispatch, unwrap_spec=['self', str, str])
+    dispatch = interp2app(W_CPPClass.dispatch, unwrap_spec=['self', 'text', 'text'])
 )
 W_CPPClass.typedef.acceptable_as_base_class = False
 
@@ -984,32 +1009,30 @@ class W_ComplexCPPClass(W_CPPClass):
 
 W_ComplexCPPClass.typedef = TypeDef(
     'ComplexCPPClass',
-    type_name = interp_attrproperty('name', W_CPPClass),
+    type_name = interp_attrproperty('name', W_CPPClass, wrapfn="newtext"),
     get_base_names = interp2app(W_ComplexCPPClass.get_base_names),
     get_method_names = interp2app(W_ComplexCPPClass.get_method_names),
-    get_overload = interp2app(W_ComplexCPPClass.get_overload, unwrap_spec=['self', str]),
+    get_overload = interp2app(W_ComplexCPPClass.get_overload, unwrap_spec=['self', 'text']),
     get_datamember_names = interp2app(W_ComplexCPPClass.get_datamember_names),
-    get_datamember = interp2app(W_ComplexCPPClass.get_datamember, unwrap_spec=['self', str]),
+    get_datamember = interp2app(W_ComplexCPPClass.get_datamember, unwrap_spec=['self', 'text']),
     is_namespace = interp2app(W_ComplexCPPClass.is_namespace),
-    dispatch = interp2app(W_CPPClass.dispatch, unwrap_spec=['self', str, str])
+    dispatch = interp2app(W_CPPClass.dispatch, unwrap_spec=['self', 'text', 'text'])
 )
 W_ComplexCPPClass.typedef.acceptable_as_base_class = False
 
 
 class W_CPPTemplateType(W_Root):
-    _attrs_ = ['space', 'name', 'handle']
-    _immutable_fields = ['name', 'handle']
+    _attrs_ = ['space', 'name']
+    _immutable_fields = ['name']
 
-    def __init__(self, space, name, opaque_handle):
+    def __init__(self, space, name):
         self.space = space
         self.name = name
-        assert lltype.typeOf(opaque_handle) == capi.C_TYPE
-        self.handle = opaque_handle
 
     @unwrap_spec(args_w='args_w')
     def __call__(self, args_w):
         # TODO: this is broken but unused (see pythonify.py)
-        fullname = "".join([self.name, '<', self.space.str_w(args_w[0]), '>'])
+        fullname = "".join([self.name, '<', self.space.text_w(args_w[0]), '>'])
         return scope_byname(self.space, fullname)
 
 W_CPPTemplateType.typedef = TypeDef(
@@ -1038,7 +1061,8 @@ class W_CPPInstance(W_Root):
         self._opt_register_finalizer()
 
     def _opt_register_finalizer(self):
-        if self.python_owns and not self.finalizer_registered:
+        if self.python_owns and not self.finalizer_registered \
+               and not hasattr(self.space, "fake"):
             self.register_finalizer(self.space)
             self.finalizer_registered = True
 
@@ -1049,7 +1073,7 @@ class W_CPPInstance(W_Root):
 
     # allow user to determine ownership rules on a per object level
     def fget_python_owns(self, space):
-        return space.wrap(self.python_owns)
+        return space.newbool(self.python_owns)
 
     @unwrap_spec(value=bool)
     def fset_python_owns(self, space, value):
@@ -1068,7 +1092,7 @@ class W_CPPInstance(W_Root):
 
     def _get_as_builtin(self):
         try:
-            return self.space.call_method(self.space.wrap(self), "_cppyy_as_builtin")
+            return self.space.call_method(self, "_cppyy_as_builtin")
         except OperationError as e:
             if not (e.match(self.space, self.space.w_TypeError) or
                     e.match(self.space, self.space.w_AttributeError)):
@@ -1078,20 +1102,17 @@ class W_CPPInstance(W_Root):
         return None
 
     def instance__init__(self, args_w):
-        try:
-            constructor_overload = self.cppclass.get_overload(self.cppclass.name)
-            constructor_overload.call(self, args_w)
-        except OperationError as e:
-            if not e.match(self.space, self.space.w_AttributeError):
-                raise
+        if capi.c_is_abstract(self.space, self.cppclass.handle):
             raise oefmt(self.space.w_TypeError,
                         "cannot instantiate abstract class '%s'",
                         self.cppclass.name)
-
+        constructor_overload = self.cppclass.get_overload(self.cppclass.name)
+        constructor_overload.call(self, args_w)
+ 
     def instance__eq__(self, w_other):
         # special case: if other is None, compare pointer-style
         if self.space.is_w(w_other, self.space.w_None):
-            return self.space.wrap(not self._rawobject)
+            return self.space.newbool(not self._rawobject)
 
         # get here if no class-specific overloaded operator is available, try to
         # find a global overload in gbl, in __gnu_cxx (for iterators), or in the
@@ -1099,9 +1120,10 @@ class W_CPPInstance(W_Root):
         try:
             # TODO: expecting w_other to be an W_CPPInstance is too limiting
             other = self.space.interp_w(W_CPPInstance, w_other, can_be_None=False)
-            for name in ["", "__gnu_cxx"]:
+            for name in ["", "__gnu_cxx", "__1"]:
                 nss = scope_byname(self.space, name)
-                meth_idx = capi.c_get_global_operator(self.space, nss, self.cppclass, other.cppclass, "==")
+                meth_idx = capi.c_get_global_operator(
+                    self.space, nss, self.cppclass, other.cppclass, "operator==")
                 if meth_idx != -1:
                     f = nss._make_cppfunction("operator==", meth_idx)
                     ol = W_CPPOverload(self.space, nss, [f])
@@ -1121,7 +1143,7 @@ class W_CPPInstance(W_Root):
         # the first data member in a struct and the struct have the same address)
         other = self.space.interp_w(W_CPPInstance, w_other, can_be_None=False)  # TODO: factor out
         iseq = (self._rawobject == other._rawobject) and (self.cppclass == other.cppclass)
-        return self.space.wrap(iseq)
+        return self.space.newbool(iseq)
 
     def instance__ne__(self, w_other):
         return self.space.not_(self.instance__eq__(w_other))
@@ -1149,7 +1171,7 @@ class W_CPPInstance(W_Root):
         w_as_builtin = self._get_as_builtin()
         if w_as_builtin is not None:
             return self.space.repr(w_as_builtin)
-        return self.space.wrap("<%s object at 0x%x>" %
+        return self.space.newtext("<%s object at 0x%x>" %
                                (self.cppclass.name, rffi.cast(rffi.ULONG, self.get_rawobject())))
 
     def destruct(self):
@@ -1164,7 +1186,7 @@ class W_CPPInstance(W_Root):
 
 W_CPPInstance.typedef = TypeDef(
     'CPPInstance',
-    cppclass = interp_attrproperty('cppclass', cls=W_CPPInstance),
+    cppclass = interp_attrproperty_w('cppclass', cls=W_CPPInstance),
     _python_owns = GetSetProperty(W_CPPInstance.fget_python_owns, W_CPPInstance.fset_python_owns),
     __init__ = interp2app(W_CPPInstance.instance__init__),
     __eq__ = interp2app(W_CPPInstance.instance__eq__),
@@ -1215,12 +1237,12 @@ def get_pythonized_cppclass(space, handle):
     except KeyError:
         final_name = capi.c_scoped_final_name(space, handle)
         # the callback will cache the class by calling register_class
-        w_pycppclass = space.call_function(state.w_clgen_callback, space.wrap(final_name))
+        w_pycppclass = space.call_function(state.w_clgen_callback, space.newtext(final_name))
     return w_pycppclass
 
 def get_interface_func(space, w_callable, npar):
     state = space.fromcache(State)
-    return space.call_function(state.w_fngen_callback, w_callable, space.wrap(npar))
+    return space.call_function(state.w_fngen_callback, w_callable, space.newint(npar))
 
 def wrap_cppobject(space, rawobject, cppclass,
                    do_cast=True, python_owns=False, is_ref=False, fresh=False):
@@ -1235,7 +1257,7 @@ def wrap_cppobject(space, rawobject, cppclass,
                 w_pycppclass = get_pythonized_cppclass(space, actual)
                 offset = capi.c_base_offset1(space, actual, cppclass, rawobject, -1)
                 rawobject = capi.direct_ptradd(rawobject, offset)
-                w_cppclass = space.findattr(w_pycppclass, space.wrap("_cpp_proxy"))
+                w_cppclass = space.findattr(w_pycppclass, space.newtext("_cpp_proxy"))
                 cppclass = space.interp_w(W_CPPClass, w_cppclass, can_be_None=False)
             except Exception:
                 # failed to locate/build the derived class, so stick to the base (note
@@ -1272,7 +1294,7 @@ def _addressof(space, w_obj):
 def addressof(space, w_obj):
     """Takes a bound C++ instance or array, returns the raw address."""
     address = _addressof(space, w_obj)
-    return space.wrap(address)
+    return space.newlong(address)
 
 @unwrap_spec(owns=bool, cast=bool)
 def bind_object(space, w_obj, w_pycppclass, owns=False, cast=False):
@@ -1283,11 +1305,11 @@ def bind_object(space, w_obj, w_pycppclass, owns=False, cast=False):
     except Exception:
         # accept integer value as address
         rawobject = rffi.cast(capi.C_OBJECT, space.uint_w(w_obj))
-    w_cppclass = space.findattr(w_pycppclass, space.wrap("_cpp_proxy"))
+    w_cppclass = space.findattr(w_pycppclass, space.newtext("_cpp_proxy"))
     if not w_cppclass:
-        w_cppclass = scope_byname(space, space.str_w(w_pycppclass))
+        w_cppclass = scope_byname(space, space.text_w(w_pycppclass))
         if not w_cppclass:
             raise oefmt(space.w_TypeError,
-                        "no such class: %s", space.str_w(w_pycppclass))
+                        "no such class: %s", space.text_w(w_pycppclass))
     cppclass = space.interp_w(W_CPPClass, w_cppclass, can_be_None=False)
     return wrap_cppobject(space, rawobject, cppclass, do_cast=cast, python_owns=owns)
