@@ -1,16 +1,5 @@
 #ifndef Py_DESCROBJECT_H
 #define Py_DESCROBJECT_H
-typedef PyObject *(*getter)(PyObject *, void *);
-typedef int (*setter)(PyObject *, PyObject *, void *);
-
-typedef struct PyGetSetDef {
-	char *name;
-	getter get;
-	setter set;
-	char *doc;
-	void *closure;
-} PyGetSetDef;
-
 
 #define PyDescr_COMMON \
     PyObject_HEAD \
