@@ -19,7 +19,7 @@ def PySequence_Repeat(space, w_obj, count):
     """Return the result of repeating sequence object o count times, or NULL on
     failure.  This is the equivalent of the Python expression o * count.
     """
-    return space.mul(w_obj, space.wrap(count))
+    return space.mul(w_obj, space.newint(count))
 
 @cpython_api([PyObject], rffi.INT_real, error=CANNOT_FAIL)
 def PySequence_Check(space, w_obj):
@@ -43,16 +43,20 @@ def PySequence_Length(space, w_obj):
 def PySequence_Fast(space, w_obj, m):
     """Returns the sequence o as a tuple, unless it is already a tuple or list, in
     which case o is returned.  Use PySequence_Fast_GET_ITEM() to access the
-    members of the result.  Returns NULL on failure.  If the object is not a
-    sequence, raises TypeError with m as the message text."""
+    members of the result.  Returns NULL on failure.  If the object cannot be
+    converted to a sequence, and raises a TypeError, raise a new TypeError with
+    m as the message text. If the conversion otherwise, fails, reraise the
+    original exception"""
     if isinstance(w_obj, W_ListObject):
         # make sure we can return a borrowed obj from PySequence_Fast_GET_ITEM    
         w_obj.convert_to_cpy_strategy(space)
         return w_obj
     try:
         return W_ListObject.newlist_cpyext(space, space.listview(w_obj))
-    except OperationError:
-        raise OperationError(space.w_TypeError, space.wrap(rffi.charp2str(m)))
+    except OperationError as e:
+        if e.match(space, space.w_TypeError):
+            raise OperationError(space.w_TypeError, space.newtext(rffi.charp2str(m)))
+        raise e
 
 @cpython_api([rffi.VOIDP, Py_ssize_t], PyObject, result_borrowed=True)
 def PySequence_Fast_GET_ITEM(space, w_obj, index):
@@ -103,20 +107,20 @@ def PySequence_Fast_ITEMS(space, w_obj):
 def PySequence_GetSlice(space, w_obj, start, end):
     """Return the slice of sequence object o between i1 and i2, or NULL on
     failure. This is the equivalent of the Python expression o[i1:i2]."""
-    return space.getslice(w_obj, space.wrap(start), space.wrap(end))
+    return space.getslice(w_obj, space.newint(start), space.newint(end))
 
 @cpython_api([PyObject, Py_ssize_t, Py_ssize_t, PyObject], rffi.INT_real, error=-1)
 def PySequence_SetSlice(space, w_obj, start, end, w_value):
     """Assign the sequence object v to the slice in sequence object o from i1 to
     i2.  This is the equivalent of the Python statement o[i1:i2] = v."""
-    space.setslice(w_obj, space.wrap(start), space.wrap(end), w_value)
+    space.setslice(w_obj, space.newint(start), space.newint(end), w_value)
     return 0
 
 @cpython_api([PyObject, Py_ssize_t, Py_ssize_t], rffi.INT_real, error=-1)
 def PySequence_DelSlice(space, w_obj, start, end):
     """Delete the slice in sequence object o from i1 to i2.  Returns -1 on
     failure.  This is the equivalent of the Python statement del o[i1:i2]."""
-    space.delslice(w_obj, space.wrap(start), space.wrap(end))
+    space.delslice(w_obj, space.newint(start), space.newint(end))
     return 0
 
 @cpython_api([rffi.VOIDP, Py_ssize_t], PyObject)
@@ -128,7 +132,7 @@ def PySequence_ITEM(space, w_obj, i):
 
     This function used an int type for i. This might require
     changes in your code for properly supporting 64-bit systems."""
-    return space.getitem(w_obj, space.wrap(i))
+    return space.getitem(w_obj, space.newint(i))
 
 @cpython_api([PyObject, Py_ssize_t], PyObject)
 def PySequence_GetItem(space, w_obj, i):
@@ -171,7 +175,7 @@ def PySequence_InPlaceRepeat(space, w_o, count):
 
     This function used an int type for count. This might require
     changes in your code for properly supporting 64-bit systems."""
-    return space.inplace_mul(w_o, space.wrap(count))
+    return space.inplace_mul(w_o, space.newint(count))
 
 
 @cpython_api([PyObject, PyObject], rffi.INT_real, error=-1)
@@ -199,14 +203,14 @@ def PySequence_SetItem(space, w_o, i, w_v):
     if PyDict_Check(space, w_o) or not PySequence_Check(space, w_o):
         raise oefmt(space.w_TypeError,
                     "'%T' object does not support item assignment", w_o)
-    space.setitem(w_o, space.wrap(i), w_v)
+    space.setitem(w_o, space.newint(i), w_v)
     return 0
 
 @cpython_api([PyObject, Py_ssize_t], rffi.INT_real, error=-1)
 def PySequence_DelItem(space, w_o, i):
     """Delete the ith element of object o.  Returns -1 on failure.  This is the
     equivalent of the Python statement del o[i]."""
-    space.delitem(w_o, space.wrap(i))
+    space.delitem(w_o, space.newint(i))
     return 0
 
 @cpython_api([PyObject, PyObject], Py_ssize_t, error=-1)
@@ -226,7 +230,7 @@ def PySequence_Index(space, w_seq, w_obj):
             if e.match(space, space.w_StopIteration):
                 break
             raise
-        if space.is_true(space.eq(w_next, w_obj)):
+        if space.eq_w(w_next, w_obj):
             return idx
         idx += 1
 
