@@ -160,6 +160,28 @@ foo_setattro(fooobject *self, PyObject *name, PyObject *value)
     return PyObject_GenericSetAttr((PyObject *)self, name, value);
 }
 
+static PyObject *
+new_fooType(PyTypeObject * t, PyObject *args, PyObject *kwds)
+{
+    PyObject * o;
+    /* copied from numpy scalartypes.c for inherited classes */
+    if (t->tp_bases && (PyTuple_GET_SIZE(t->tp_bases) > 1))
+    {
+        PyTypeObject *sup; 
+        /* We are inheriting from a Python type as well so 
+           give it first dibs on conversion */
+        sup = (PyTypeObject *)PyTuple_GET_ITEM(t->tp_bases, 1);
+        /* Prevent recursion */ 
+        if (new_fooType != sup->tp_new) 
+        {
+            o = sup->tp_new(t, args, kwds);
+            return o;
+        }
+    }
+    o = t->tp_alloc(t, 0);
+    return o;   
+};
+
 static PyMemberDef foo_members[] = {
     {"int_member", T_INT, offsetof(fooobject, foo), 0,
      "A helpful docstring."},
@@ -259,8 +281,7 @@ static PyMethodDef UnicodeSubclass_methods[] = {
 };
 
 PyTypeObject UnicodeSubtype = {
-    PyObject_HEAD_INIT(NULL)
-    0,
+    PyVarObject_HEAD_INIT(NULL, 0)
     "foo.fuu",
     sizeof(UnicodeSubclassObject),
     0,
@@ -318,8 +339,7 @@ PyTypeObject UnicodeSubtype = {
 };
 
 PyTypeObject UnicodeSubtype2 = {
-    PyObject_HEAD_INIT(NULL)
-    0,
+    PyVarObject_HEAD_INIT(NULL, 0)
     "foo.fuu2",
     sizeof(UnicodeSubclassObject),
     0,
@@ -377,8 +397,7 @@ PyTypeObject UnicodeSubtype2 = {
 };
 
 PyTypeObject UnicodeSubtype3 = {
-    PyObject_HEAD_INIT(NULL)
-    0,
+    PyVarObject_HEAD_INIT(NULL, 0)
     "foo.fuu3",
     sizeof(UnicodeSubclassObject)
 };
@@ -386,8 +405,7 @@ PyTypeObject UnicodeSubtype3 = {
 /* A Metatype */
 
 PyTypeObject MetaType = {
-    PyObject_HEAD_INIT(NULL)
-    0,
+    PyVarObject_HEAD_INIT(NULL, 0)
     "foo.Meta",
     sizeof(PyHeapTypeObject),/*tp_basicsize*/
     0,          /*tp_itemsize*/
@@ -466,8 +484,7 @@ static int initerrtype_init(PyObject *self, PyObject *args, PyObject *kwargs) {
 
 
 PyTypeObject InitErrType = {
-    PyObject_HEAD_INIT(NULL)
-    0,
+    PyVarObject_HEAD_INIT(NULL, 0)
     "foo.InitErrType",
     sizeof(PyObject),/*tp_basicsize*/
     0,          /*tp_itemsize*/
@@ -550,8 +567,7 @@ int prop_descr_set(PyObject *self, PyObject *obj, PyObject *value)
 
 
 PyTypeObject SimplePropertyType = {
-    PyObject_HEAD_INIT(NULL)
-    0,
+    PyVarObject_HEAD_INIT(NULL, 0)
     "foo.Property",
     sizeof(PyObject),
     0,
@@ -619,14 +635,13 @@ static PyTypeObject CustomType;
 static PyObject *newCustom(PyObject *self, PyObject *args)
 {
     PyObject *obj = calloc(1, sizeof(PyObject));
-    obj->ob_type = &CustomType;
+    Py_TYPE(obj) = &CustomType;
     _Py_NewReference(obj);
     return obj;
 }
 
 static PyTypeObject CustomType = {
-    PyObject_HEAD_INIT(NULL)
-    0,
+    PyVarObject_HEAD_INIT(NULL, 0)
     "foo.Custom",            /*tp_name*/
     sizeof(PyObject),        /*tp_size*/
     0,                       /*tp_itemsize*/
@@ -635,8 +650,7 @@ static PyTypeObject CustomType = {
 };
 
 static PyTypeObject TupleLike = {
-    PyObject_HEAD_INIT(NULL)
-    0,
+    PyVarObject_HEAD_INIT(NULL, 0)
     "foo.TupleLike",         /*tp_name*/
     sizeof(PyObject),        /*tp_size*/
 };
@@ -656,6 +670,34 @@ static PyObject * is_TupleLike(PyObject *self, PyObject * t)
     return PyInt_FromLong(tf);
 }
 
+static PyTypeObject GetType1 = {
+    PyVarObject_HEAD_INIT(NULL, 0)
+    "foo.GetType1",          /*tp_name*/
+    sizeof(PyObject),        /*tp_size*/
+};
+static PyTypeObject GetType2 = {
+    PyVarObject_HEAD_INIT(NULL, 0)
+    "foo.GetType2",          /*tp_name*/
+    sizeof(PyObject),        /*tp_size*/
+};
+static PyObject *gettype1, *gettype2;
+
+static PyObject *gettype1_getattr(PyObject *self, char *name)
+{
+    char buf[200];
+    strcpy(buf, "getattr:");
+    strcat(buf, name);
+    return PyString_FromString(buf);
+}
+static PyObject *gettype2_getattro(PyObject *self, PyObject *name)
+{
+    char buf[200];
+    strcpy(buf, "getattro:");
+    strcat(buf, PyString_AS_STRING(name));
+    return PyString_FromString(buf);
+}
+
+
 /* List of functions exported by this module */
 
 static PyMethodDef foo_functions[] = {
@@ -720,6 +762,9 @@ initfoo(void)
     UnicodeSubtype2.tp_base = &UnicodeSubtype;
     MetaType.tp_base = &PyType_Type;
 
+    footype.tp_new = &new_fooType;
+    InitErrType.tp_new = PyType_GenericNew;
+
     if (PyType_Ready(&footype) < 0)
         INITERROR;
     if (PyType_Ready(&UnicodeSubtype) < 0)
@@ -736,7 +781,7 @@ initfoo(void)
     SimplePropertyType.tp_new = PyType_GenericNew;
     InitErrType.tp_new = PyType_GenericNew;
 
-    CustomType.ob_type = &MetaType;
+    Py_TYPE(&CustomType) = &MetaType;
     if (PyType_Ready(&CustomType) < 0)
         INITERROR;
 
@@ -752,6 +797,18 @@ initfoo(void)
     if (PyType_Ready(&TupleLike) < 0)
         INITERROR;
 
+    GetType1.tp_flags = Py_TPFLAGS_DEFAULT;
+    GetType1.tp_getattr = &gettype1_getattr;
+    if (PyType_Ready(&GetType1) < 0)
+        INITERROR;
+    gettype1 = PyObject_New(PyObject, &GetType1);
+
+    GetType2.tp_flags = Py_TPFLAGS_DEFAULT;
+    GetType2.tp_getattro = &gettype2_getattro;
+    if (PyType_Ready(&GetType2) < 0)
+        INITERROR;
+    gettype2 = PyObject_New(PyObject, &GetType2);
+
 
     d = PyModule_GetDict(module);
     if (d == NULL)
@@ -774,6 +831,10 @@ initfoo(void)
         INITERROR;
     if (PyDict_SetItemString(d, "TupleLike", (PyObject *) &TupleLike) < 0)
         INITERROR;
+    if (PyDict_SetItemString(d, "gettype1", gettype1) < 0)
+        INITERROR;
+    if (PyDict_SetItemString(d, "gettype2", gettype2) < 0)
+        INITERROR;
 #if PY_MAJOR_VERSION >=3
     return module;
 #endif
