@@ -974,6 +974,8 @@ class AppTestSlots(AppTestCpythonExtensionBase):
             pass
         class foo(f2, f1):
             pass
+
+        x = foo()
         assert bar.__base__ is f2
         # On cpython, the size changes.
         if '__pypy__' in sys.builtin_module_names:
@@ -1243,3 +1245,14 @@ class AppTestSlots(AppTestCpythonExtensionBase):
                 pass
             bases = module.foo(C)
             assert bases == (A, B)
+
+    def test_getattr_getattro(self):
+        module = self.import_module(name='foo')
+        assert module.gettype2.dcba == 'getattro:dcba'
+        assert (type(module.gettype2).__getattribute__(module.gettype2, 'dcBA')
+            == 'getattro:dcBA')
+        assert module.gettype1.abcd == 'getattr:abcd'
+        # GetType1 objects have a __getattribute__ method, but this
+        # doesn't call tp_getattr at all, also on CPython
+        raises(AttributeError, type(module.gettype1).__getattribute__,
+                               module.gettype1, 'dcBA')
