@@ -283,6 +283,7 @@ class AppTestStruct(object):
         assert pack(">?", False) == '\x00'
         assert pack("@?", True) == '\x01'
         assert pack("@?", False) == '\x00'
+        assert self.struct.unpack("?", 'X')[0] is True
 
     def test_transitiveness(self):
         c = 'a'
@@ -431,6 +432,20 @@ class AppTestStruct(object):
     def test_overflow(self):
         raises(self.struct.error, self.struct.pack, 'i', 1<<65)
 
+    def test_unpack_fits_into_int(self):
+        import sys
+        for fmt in 'ILQq':
+            # check that we return an int, if it fits
+            buf = self.struct.pack(fmt, 42)
+            val, = self.struct.unpack(fmt, buf)
+            assert val == 42
+            assert type(val) is int
+        #
+        # check that we return a long, if it doesn't fit into an int
+        buf = self.struct.pack('Q', sys.maxint+1)
+        val, = self.struct.unpack('Q', buf)
+        assert val == sys.maxint+1
+        assert type(val) is long
 
 class AppTestStructBuffer(object):
     spaceconfig = dict(usemodules=['struct', '__pypy__'])
