@@ -1,5 +1,6 @@
 """The builtin str implementation"""
 
+from rpython.rlib import jit
 from rpython.rlib.jit import we_are_jitted
 from rpython.rlib.objectmodel import (
     compute_hash, compute_unique_id, import_from_mixin)
@@ -31,8 +32,8 @@ class W_AbstractBytesObject(W_Root):
             return True
         if self.user_overridden_class or w_other.user_overridden_class:
             return False
-        s1 = space.str_w(self)
-        s2 = space.str_w(w_other)
+        s1 = space.bytes_w(self)
+        s2 = space.bytes_w(w_other)
         if len(s2) > 1:
             return s1 is s2
         else:            # strings of len <= 1 are unique-ified
@@ -41,7 +42,7 @@ class W_AbstractBytesObject(W_Root):
     def immutable_unique_id(self, space):
         if self.user_overridden_class:
             return None
-        s = space.str_w(self)
+        s = space.bytes_w(self)
         if len(s) > 1:
             uid = compute_unique_id(s)
         else:            # strings of len <= 1 are unique-ified
@@ -50,7 +51,7 @@ class W_AbstractBytesObject(W_Root):
             else:
                 base = 256           # empty string: base value 256
             uid = (base << IDTAG_SHIFT) | IDTAG_SPECIAL
-        return space.wrap(uid)
+        return space.newint(uid)
 
     def unicode_w(self, space):
         # Use the default encoding.
@@ -478,7 +479,7 @@ class W_BytesObject(W_AbstractBytesObject):
             raise oefmt(space.w_TypeError,
                         "ord() expected a character, but string of length %d "
                         "found", len(self._value))
-        return space.wrap(ord(self._value[0]))
+        return space.newint(ord(self._value[0]))
 
     def _new(self, value):
         return W_BytesObject(value)
@@ -501,9 +502,12 @@ class W_BytesObject(W_AbstractBytesObject):
                 isinstance(w_other, W_UnicodeObject))
 
     @staticmethod
-    def _op_val(space, w_other):
+    def _op_val(space, w_other, strict=None):
+        if strict and not space.isinstance_w(w_other, space.w_bytes):
+            raise oefmt(space.w_TypeError,
+                "%s arg must be None, str or unicode", strict)
         try:
-            return space.str_w(w_other)
+            return space.bytes_w(w_other)
         except OperationError as e:
             if not e.match(space, space.w_TypeError):
                 raise
@@ -566,9 +570,9 @@ class W_BytesObject(W_AbstractBytesObject):
         # NB. the default value of w_object is really a *wrapped* empty string:
         #     there is gateway magic at work
         w_obj = space.str(w_object)
-        if space.is_w(w_stringtype, space.w_str):
+        if space.is_w(w_stringtype, space.w_bytes):
             return w_obj  # XXX might be reworked when space.str() typechecks
-        value = space.str_w(w_obj)
+        value = space.bytes_w(w_obj)
         w_obj = space.allocate_instance(W_BytesObject, w_stringtype)
         W_BytesObject.__init__(w_obj, value)
         return w_obj
@@ -578,7 +582,7 @@ class W_BytesObject(W_AbstractBytesObject):
         quote = "'"
         if quote in s and '"' not in s:
             quote = '"'
-        return space.wrap(string_escape_encode(s, quote))
+        return space.newtext(string_escape_encode(s, quote))
 
     def descr_str(self, space):
         if type(self) is W_BytesObject:
@@ -587,21 +591,24 @@ class W_BytesObject(W_AbstractBytesObject):
 
     def descr_hash(self, space):
         x = compute_hash(self._value)
-        return space.wrap(x)
+        return space.newint(x)
 
     def descr_format(self, space, __args__):
         return newformat.format_method(space, self, __args__, is_unicode=False)
 
     def descr__format__(self, space, w_format_spec):
-        if not space.isinstance_w(w_format_spec, space.w_str):
+        if not space.isinstance_w(w_format_spec, space.w_bytes):
             w_format_spec = space.str(w_format_spec)
-        spec = space.str_w(w_format_spec)
+        spec = space.bytes_w(w_format_spec)
         formatter = newformat.str_formatter(space, spec)
         return formatter.format_string(self._value)
 
     def descr_mod(self, space, w_values):
         return mod_format(space, self, w_values, do_unicode=False)
 
+    def descr_rmod(self, space, w_values):
+        return mod_format(space, w_values, self, do_unicode=False)
+
     def descr_eq(self, space, w_other):
         if space.config.objspace.std.withstrbuf:
             from pypy.objspace.std.strbufobject import W_StringBufferObject
@@ -820,11 +827,11 @@ class W_BytesObject(W_AbstractBytesObject):
         return self._StringMethods_descr_rpartition(space, w_sub)
 
     def _join_return_one(self, space, w_obj):
-        return (space.is_w(space.type(w_obj), space.w_str) or
+        return (space.is_w(space.type(w_obj), space.w_bytes) or
                 space.is_w(space.type(w_obj), space.w_unicode))
 
     def _join_check_item(self, space, w_obj):
-        if space.isinstance_w(w_obj, space.w_str):
+        if space.isinstance_w(w_obj, space.w_bytes):
             return 0
         if space.isinstance_w(w_obj, space.w_unicode):
             return 2
@@ -845,12 +852,12 @@ class W_BytesObject(W_AbstractBytesObject):
 
     def descr_formatter_parser(self, space):
         from pypy.objspace.std.newformat import str_template_formatter
-        tformat = str_template_formatter(space, space.str_w(self))
+        tformat = str_template_formatter(space, space.bytes_w(self))
         return tformat.formatter_parser()
 
     def descr_formatter_field_name_split(self, space):
         from pypy.objspace.std.newformat import str_template_formatter
-        tformat = str_template_formatter(space, space.str_w(self))
+        tformat = str_template_formatter(space, space.bytes_w(self))
         return tformat.formatter_field_name_split()
 
 
@@ -934,6 +941,7 @@ W_BytesObject.typedef = TypeDef(
     format = interpindirect2app(W_BytesObject.descr_format),
     __format__ = interpindirect2app(W_BytesObject.descr__format__),
     __mod__ = interpindirect2app(W_BytesObject.descr_mod),
+    __rmod__ = interpindirect2app(W_BytesObject.descr_rmod),
     __getnewargs__ = interpindirect2app(
         W_AbstractBytesObject.descr_getnewargs),
     _formatter_parser = interp2app(W_BytesObject.descr_formatter_parser),
@@ -943,6 +951,7 @@ W_BytesObject.typedef = TypeDef(
 W_BytesObject.typedef.flag_sequence_bug_compat = True
 
 
+@jit.elidable
 def string_escape_encode(s, quote):
     buf = StringBuilder(len(s) + 2)
 
