@@ -8,6 +8,7 @@ from rpython.rlib import rstring, runicode, rlocale, rfloat, jit
 from rpython.rlib.objectmodel import specialize
 from rpython.rlib.rfloat import copysign, formatd
 from rpython.rlib.rarithmetic import r_uint, intmask
+from pypy.interpreter.signature import Signature
 
 
 @specialize.argtype(1)
@@ -40,19 +41,39 @@ ANS_AUTO = 2
 ANS_MANUAL = 3
 
 
-def make_template_formatting_class():
+format_signature = Signature([], 'args', 'kwargs')
+
+
+def make_template_formatting_class(for_unicode):
     class TemplateFormatter(object):
+        is_unicode = for_unicode
+
+        if for_unicode:
+            def wrap(self, u):
+                return self.space.newunicode(u)
+        else:
+            def wrap(self, s):
+                return self.space.newbytes(s)
 
         parser_list_w = None
 
-        def __init__(self, space, is_unicode, template):
+        def __init__(self, space, template):
             self.space = space
-            self.is_unicode = is_unicode
-            self.empty = u"" if is_unicode else ""
+            self.empty = u"" if self.is_unicode else ""
             self.template = template
 
         def build(self, args):
-            self.args, self.kwargs = args.unpack()
+            if self.is_unicode:
+                # for unicode, use the slower parse_obj() to get self.w_kwargs
+                # as a wrapped dictionary that may contain full-range unicode
+                # keys.  See test_non_latin1_key
+                space = self.space
+                w_args, w_kwds = args.parse_obj(None, 'format',
+                                                format_signature)
+                self.args = space.listview(w_args)
+                self.w_kwargs = w_kwds
+            else:
+                self.args, self.kwargs = args.unpack()
             self.auto_numbering = 0
             self.auto_numbering_state = ANS_INIT
             return self._build_string(0, len(self.template), 2)
@@ -99,7 +120,7 @@ def make_template_formatting_class():
                             assert end_literal > last_literal
                             literal = self.template[last_literal:end_literal]
                             w_entry = space.newtuple([
-                                space.wrap(literal),
+                                self.wrap(literal),
                                 space.w_None, space.w_None, space.w_None])
                             self.parser_list_w.append(w_entry)
                             self.last_end = i
@@ -197,17 +218,13 @@ def make_template_formatting_class():
             if index == -1:
                 kwarg = name[:i]
                 if self.is_unicode:
-                    try:
-                        arg_key = kwarg.encode("latin-1")
-                    except UnicodeEncodeError:
-                        # Not going to be found in a dict of strings.
-                        raise OperationError(space.w_KeyError, space.wrap(kwarg))
+                    w_arg = space.getitem(self.w_kwargs, self.wrap(kwarg))
                 else:
-                    arg_key = kwarg
-                try:
-                    w_arg = self.kwargs[arg_key]
-                except KeyError:
-                    raise OperationError(space.w_KeyError, space.wrap(arg_key))
+                    try:
+                        w_arg = self.kwargs[kwarg]
+                    except KeyError:
+                        raise OperationError(space.w_KeyError,
+                                             space.newtext(kwarg))
             else:
                 try:
                     w_arg = self.args[index]
@@ -233,7 +250,7 @@ def make_template_formatting_class():
                     if start == i:
                         raise oefmt(space.w_ValueError,
                                     "Empty attribute in format string")
-                    w_attr = space.wrap(name[start:i])
+                    w_attr = self.wrap(name[start:i])
                     if w_obj is not None:
                         w_obj = space.getattr(w_obj, w_attr)
                     else:
@@ -253,9 +270,9 @@ def make_template_formatting_class():
                         raise oefmt(space.w_ValueError, "Missing ']'")
                     index, reached = _parse_int(self.space, name, start, i)
                     if index != -1 and reached == i:
-                        w_item = space.wrap(index)
+                        w_item = space.newint(index)
                     else:
-                        w_item = space.wrap(name[start:i])
+                        w_item = self.wrap(name[start:i])
                     i += 1 # Skip "]"
                     if w_obj is not None:
                         w_obj = space.getitem(w_obj, w_item)
@@ -284,9 +301,9 @@ def make_template_formatting_class():
                 if stop != i:
                     index = -1
             if index >= 0:
-                w_first = space.wrap(index)
+                w_first = space.newint(index)
             else:
-                w_first = space.wrap(name[:i])
+                w_first = self.wrap(name[:i])
             #
             self.parser_list_w = []
             self._resolve_lookups(None, name, i, end)
@@ -316,11 +333,15 @@ def make_template_formatting_class():
                     space = self.space
                     startm1 = start - 1
                     assert startm1 >= self.last_end
+                    if conversion is None:
+                        w_conversion = space.w_None
+                    else:
+                        w_conversion = self.wrap(conversion)
                     w_entry = space.newtuple([
-                        space.wrap(self.template[self.last_end:startm1]),
-                        space.wrap(name),
-                        space.wrap(spec),
-                        space.wrap(conversion)])
+                        self.wrap(self.template[self.last_end:startm1]),
+                        self.wrap(name),
+                        self.wrap(spec),
+                        w_conversion])
                     self.parser_list_w.append(w_entry)
                     self.last_end = end + 1
                 return self.empty
@@ -330,8 +351,8 @@ def make_template_formatting_class():
                 w_obj = self._convert(w_obj, conversion)
             if recursive:
                 spec = self._build_string(spec_start, end, level)
-            w_rendered = self.space.format(w_obj, self.space.wrap(spec))
-            unwrapper = "unicode_w" if self.is_unicode else "str_w"
+            w_rendered = self.space.format(w_obj, self.wrap(spec))
+            unwrapper = "unicode_w" if self.is_unicode else "bytes_w"
             to_interp = getattr(self.space, unwrapper)
             return to_interp(w_rendered)
 
@@ -343,7 +364,7 @@ def make_template_formatting_class():
             space = self.space
             if self.last_end < len(self.template):
                 w_lastentry = space.newtuple([
-                    space.wrap(self.template[self.last_end:]),
+                    self.wrap(self.template[self.last_end:]),
                     space.w_None,
                     space.w_None,
                     space.w_None])
@@ -351,24 +372,18 @@ def make_template_formatting_class():
             return space.iter(space.newlist(self.parser_list_w))
     return TemplateFormatter
 
-StrTemplateFormatter = make_template_formatting_class()
-UnicodeTemplateFormatter = make_template_formatting_class()
-
-def str_template_formatter(space, template):
-    return StrTemplateFormatter(space, False, template)
-
-def unicode_template_formatter(space, template):
-    return UnicodeTemplateFormatter(space, True, template)
+str_template_formatter = make_template_formatting_class(for_unicode=False)
+unicode_template_formatter = make_template_formatting_class(for_unicode=True)
 
 
 def format_method(space, w_string, args, is_unicode):
     if is_unicode:
         template = unicode_template_formatter(space,
                                               space.unicode_w(w_string))
-        return space.wrap(template.build(args))
+        return space.newunicode(template.build(args))
     else:
-        template = str_template_formatter(space, space.str_w(w_string))
-        return space.wrap(template.build(args))
+        template = str_template_formatter(space, space.bytes_w(w_string))
+        return space.newbytes(template.build(args))
 
 
 class NumberSpec(object):
@@ -395,16 +410,23 @@ CURRENT_LOCALE = 3
 
 LONG_DIGITS = string.digits + string.ascii_lowercase
 
-def make_formatting_class():
+def make_formatting_class(for_unicode):
     class Formatter(BaseFormatter):
         """__format__ implementation for builtin types."""
 
+        if for_unicode:
+            def wrap(self, u):
+                return self.space.newunicode(u)
+        else:
+            def wrap(self, s):
+                return self.space.newbytes(s)
+
+        is_unicode = for_unicode
         _grouped_digits = None
 
-        def __init__(self, space, is_unicode, spec):
+        def __init__(self, space, spec):
             self.space = space
-            self.is_unicode = is_unicode
-            self.empty = u"" if is_unicode else ""
+            self.empty = u"" if self.is_unicode else ""
             self.spec = spec
 
         def _is_alignment(self, c):
@@ -541,7 +563,7 @@ def make_formatting_class():
         def format_string(self, string):
             space = self.space
             if self._parse_spec("s", "<"):
-                return space.wrap(string)
+                return self.wrap(string)
             if self._type != "s":
                 self._unknown_presentation("string")
             if self._sign != "\0":
@@ -562,7 +584,7 @@ def make_formatting_class():
                 length = precision
                 string = string[:precision]
             self._calc_padding(string, length)
-            return space.wrap(self._pad(string))
+            return self.wrap(self._pad(string))
 
         def _get_locale(self, tp):
             if tp == "n":
@@ -761,6 +783,11 @@ def make_formatting_class():
                     raise oefmt(space.w_ValueError,
                                 "sign not allowed with 'c' presentation type")
                 value = space.int_w(w_num)
+                max_char = runicode.MAXUNICODE if self.is_unicode else 0xFF
+                if not (0 <= value <= max_char):
+                    raise oefmt(space.w_OverflowError,
+                                "%%c arg not in range(%s)",
+                                hex(max_char))
                 if self.is_unicode:
                     result = runicode.UNICHR(value)
                 else:
@@ -805,8 +832,8 @@ def make_formatting_class():
                                         n_remainder, False, result)
             fill = self._fill_char
             upper = self._type == "X"
-            return self.space.wrap(self._fill_number(spec, result, to_numeric,
-                                     to_prefix, fill, to_remainder, upper))
+            return self.wrap(self._fill_number(spec, result, to_numeric,
+                             to_prefix, fill, to_remainder, upper))
 
         def _long_to_base(self, base, value):
             prefix = ""
@@ -954,8 +981,8 @@ def make_formatting_class():
             spec = self._calc_num_width(0, sign, to_number, n_digits,
                                         n_remainder, have_dec_point, digits)
             fill = self._fill_char
-            return self.space.wrap(self._fill_number(spec, digits, to_number, 0,
-                                      fill, to_remainder, False))
+            return self.wrap(self._fill_number(spec, digits, to_number, 0,
+                             fill, to_remainder, False))
 
         def format_float(self, w_float):
             space = self.space
@@ -1116,7 +1143,7 @@ def make_formatting_class():
             #add right padding
             out.append_multiple_char(fill, self._right_pad)
 
-            return self.space.wrap(out.build())
+            return self.wrap(out.build())
 
 
         def format_complex(self, w_complex):
@@ -1138,15 +1165,8 @@ def make_formatting_class():
             self._unknown_presentation("complex")
     return Formatter
 
-StrFormatter = make_formatting_class()
-UnicodeFormatter = make_formatting_class()
-
-
-def unicode_formatter(space, spec):
-    return StrFormatter(space, True, spec)
-
-def str_formatter(space, spec):
-    return UnicodeFormatter(space, False, spec)
+str_formatter = make_formatting_class(for_unicode=False)
+unicode_formatter = make_formatting_class(for_unicode=True)
 
 
 @specialize.arg(2)
@@ -1155,5 +1175,5 @@ def run_formatter(space, w_format_spec, meth, *args):
         formatter = unicode_formatter(space, space.unicode_w(w_format_spec))
         return getattr(formatter, meth)(*args)
     else:
-        formatter = str_formatter(space, space.str_w(w_format_spec))
+        formatter = str_formatter(space, space.bytes_w(w_format_spec))
         return getattr(formatter, meth)(*args)
