@@ -8,7 +8,7 @@ from pypy.objspace.descroperation import DescrOperation, raiseattrerror
 from rpython.rlib.objectmodel import instantiate, specialize, is_annotation_constant
 from rpython.rlib.debug import make_sure_not_resized
 from rpython.rlib.rarithmetic import base_int, widen, is_valid_int
-from rpython.rlib.objectmodel import import_from_mixin
+from rpython.rlib.objectmodel import import_from_mixin, enforceargs, not_rpython
 from rpython.rlib import jit
 
 # Object imports
@@ -39,8 +39,9 @@ class StdObjSpace(ObjSpace):
     library in Restricted Python."""
     import_from_mixin(DescrOperation)
 
+    @not_rpython
     def initialize(self):
-        """NOT_RPYTHON: only for initializing the space
+        """only for initializing the space
 
         Setup all the object types and implementations.
         """
@@ -88,8 +89,15 @@ class StdObjSpace(ObjSpace):
         for typedef, cls in builtin_type_classes.items():
             w_type = self.gettypeobject(typedef)
             self.builtin_types[typedef.name] = w_type
-            setattr(self, 'w_' + typedef.name, w_type)
+            name = typedef.name
+            # we don't expose 'space.w_str' at all, to avoid confusion
+            # with Python 3.  Instead, in Python 2, it becomes
+            # space.w_bytes (or space.w_text).
+            if name == 'str':
+                name = 'bytes'
+            setattr(self, 'w_' + name, w_type)
             self._interplevel_classes[w_type] = cls
+        self.w_text = self.w_bytes   # 'space.w_text' is w_unicode on Py3
         self.w_dict.flag_map_or_seq = 'M'
         self.builtin_types["NotImplemented"] = self.w_NotImplemented
         self.builtin_types["Ellipsis"] = self.w_Ellipsis
@@ -128,15 +136,12 @@ class StdObjSpace(ObjSpace):
         assert typedef is not None
         return self.fromcache(TypeCache).getorbuild(typedef)
 
+    @not_rpython # only for tests
     @specialize.argtype(1)
     def wrap(self, x):
-        "Wraps the Python value 'x' into one of the wrapper classes."
-        # You might notice that this function is rather conspicuously
-        # not RPython.  We can get away with this because the function
-        # is specialized (see after the function body).  Also worth
-        # noting is that the isinstance's involving integer types
-        # behave rather differently to how you might expect during
-        # annotation (see pypy/annotation/builtin.py)
+        """ Wraps the Python value 'x' into one of the wrapper classes. This
+        should only be used for tests, in real code you need to use the
+        explicit new* methods."""
         if x is None:
             return self.w_None
         if isinstance(x, OperationError):
@@ -148,39 +153,29 @@ class StdObjSpace(ObjSpace):
             else:
                 return self.newint(x)
         if isinstance(x, str):
-            return self.newbytes(x)
+            return self.newtext(x)
         if isinstance(x, unicode):
             return self.newunicode(x)
         if isinstance(x, float):
             return W_FloatObject(x)
         if isinstance(x, W_Root):
-            w_result = x.__spacebind__(self)
+            w_result = x.spacebind(self)
             #print 'wrapping', x, '->', w_result
             return w_result
         if isinstance(x, base_int):
-            if self.config.objspace.std.withsmalllong:
-                from pypy.objspace.std.smalllongobject import W_SmallLongObject
-                from rpython.rlib.rarithmetic import r_longlong, r_ulonglong
-                from rpython.rlib.rarithmetic import longlongmax
-                if (not isinstance(x, r_ulonglong)
-                    or x <= r_ulonglong(longlongmax)):
-                    return W_SmallLongObject(r_longlong(x))
-            x = widen(x)
-            if isinstance(x, int):
-                return self.newint(x)
-            else:
-                return W_LongObject.fromrarith_int(x)
-        return self._wrap_not_rpython(x)
-
-    def _wrap_not_rpython(self, x):
-        "NOT_RPYTHON"
-        # _____ this code is here to support testing only _____
+            return self.newint(x)
 
         # we might get there in non-translated versions if 'x' is
         # a long that fits the correct range.
         if is_valid_int(x):
             return self.newint(x)
 
+        return self._wrap_not_rpython(x)
+
+    def _wrap_not_rpython(self, x):
+        "NOT_RPYTHON"
+        # _____ this code is here to support testing only _____
+
         # wrap() of a container works on CPython, but the code is
         # not RPython.  Don't use -- it is kept around mostly for tests.
         # Use instead newdict(), newlist(), newtuple().
@@ -223,15 +218,15 @@ class StdObjSpace(ObjSpace):
             self.wrap("refusing to wrap cpython value %r" % (x,))
         )
 
+    @not_rpython
     def wrap_exception_cls(self, x):
-        """NOT_RPYTHON"""
         if hasattr(self, 'w_' + x.__name__):
             w_result = getattr(self, 'w_' + x.__name__)
             return w_result
         return None
 
+    @not_rpython
     def wraplong(self, x):
-        "NOT_RPYTHON"
         if self.config.objspace.std.withsmalllong:
             from rpython.rlib.rarithmetic import r_longlong
             try:
@@ -244,14 +239,25 @@ class StdObjSpace(ObjSpace):
                 return W_SmallLongObject(rx)
         return W_LongObject.fromlong(x)
 
+    @not_rpython
     def unwrap(self, w_obj):
-        """NOT_RPYTHON"""
         # _____ this code is here to support testing only _____
         if isinstance(w_obj, W_Root):
             return w_obj.unwrap(self)
         raise TypeError("cannot unwrap: %r" % w_obj)
 
+    @specialize.argtype(1)
     def newint(self, intval):
+        if self.config.objspace.std.withsmalllong and isinstance(intval, base_int):
+            from pypy.objspace.std.smalllongobject import W_SmallLongObject
+            from rpython.rlib.rarithmetic import r_longlong, r_ulonglong
+            from rpython.rlib.rarithmetic import longlongmax
+            if (not isinstance(intval, r_ulonglong)
+                or intval <= r_ulonglong(longlongmax)):
+                return W_SmallLongObject(r_longlong(intval))
+        intval = widen(intval)
+        if not isinstance(intval, int):
+            return W_LongObject.fromrarith_int(intval)
         return wrapint(self, intval)
 
     def newfloat(self, floatval):
@@ -270,6 +276,10 @@ class StdObjSpace(ObjSpace):
             return W_SmallLongObject.fromint(val)
         return W_LongObject.fromint(self, val)
 
+    @specialize.argtype(1)
+    def newlong_from_rarith_int(self, val): # val is an rarithmetic type 
+        return W_LongObject.fromrarith_int(val)
+
     def newlong_from_rbigint(self, val):
         return newlong(self, val)
 
@@ -286,6 +296,8 @@ class StdObjSpace(ObjSpace):
     def newlist_bytes(self, list_s):
         return W_ListObject.newlist_bytes(self, list_s)
 
+    newlist_text = newlist_bytes
+
     def newlist_unicode(self, list_u):
         return W_ListObject.newlist_unicode(self, list_u)
 
@@ -317,13 +329,30 @@ class StdObjSpace(ObjSpace):
     def newseqiter(self, w_obj):
         return W_SeqIterObject(w_obj)
 
-    def newbuffer(self, w_obj):
-        return W_Buffer(w_obj)
+    def newbuffer(self, obj):
+        ret = W_Buffer(obj)
+        return ret
 
     def newbytes(self, s):
+        assert isinstance(s, str)
         return W_BytesObject(s)
 
+    def newtext(self, s):
+        assert isinstance(s, str)
+        return W_BytesObject(s) # Python3 this is unicode
+
+    def newtext_or_none(self, s):
+        if s is None:
+            return self.w_None
+        return self.newtext(s)
+
+    def newfilename(self, s):
+        assert isinstance(s, str) # on pypy3, this decodes the byte string
+        return W_BytesObject(s)   # with the filesystem encoding
+
     def newunicode(self, uni):
+        assert uni is not None
+        assert isinstance(uni, unicode)
         return W_UnicodeObject(uni)
 
     def type(self, w_obj):
@@ -335,6 +364,10 @@ class StdObjSpace(ObjSpace):
         return w_type.lookup(name)
     lookup._annspecialcase_ = 'specialize:lookup'
 
+    def lookup_in_type(self, w_type, name):
+        w_src, w_descr = self.lookup_in_type_where(w_type, name)
+        return w_descr
+
     def lookup_in_type_where(self, w_type, name):
         return w_type.lookup_where(name)
     lookup_in_type_where._annspecialcase_ = 'specialize:lookup_in_type_where'
@@ -347,6 +380,7 @@ class StdObjSpace(ObjSpace):
         assert isinstance(w_starttype, W_TypeObject)
         return w_type.lookup_starting_at(w_starttype, name)
 
+    @specialize.arg(1)
     def allocate_instance(self, cls, w_subtype):
         """Allocate the memory needed for an instance of an internal or
         user-defined type, without actually __init__ializing the instance."""
@@ -371,7 +405,6 @@ class StdObjSpace(ObjSpace):
                         "%N.__new__(%N): only for the type %N",
                         w_type, w_subtype, w_type)
         return instance
-    allocate_instance._annspecialcase_ = "specialize:arg(1)"
 
     # two following functions are almost identical, but in fact they
     # have different return type. First one is a resizable list, second
@@ -445,7 +478,7 @@ class StdObjSpace(ObjSpace):
             return w_obj.listview_bytes()
         if type(w_obj) is W_SetObject or type(w_obj) is W_FrozensetObject:
             return w_obj.listview_bytes()
-        if isinstance(w_obj, W_BytesObject) and self._uses_no_iter(w_obj):
+        if isinstance(w_obj, W_BytesObject) and self._str_uses_no_iter(w_obj):
             return w_obj.listview_bytes()
         if isinstance(w_obj, W_ListObject) and self._uses_list_iter(w_obj):
             return w_obj.getitems_bytes()
@@ -460,7 +493,7 @@ class StdObjSpace(ObjSpace):
             return w_obj.listview_unicode()
         if type(w_obj) is W_SetObject or type(w_obj) is W_FrozensetObject:
             return w_obj.listview_unicode()
-        if isinstance(w_obj, W_UnicodeObject) and self._uses_no_iter(w_obj):
+        if isinstance(w_obj, W_UnicodeObject) and self._uni_uses_no_iter(w_obj):
             return w_obj.listview_unicode()
         if isinstance(w_obj, W_ListObject) and self._uses_list_iter(w_obj):
             return w_obj.getitems_unicode()
@@ -504,14 +537,21 @@ class StdObjSpace(ObjSpace):
         from pypy.objspace.descroperation import tuple_iter
         return self.lookup(w_obj, '__iter__') is tuple_iter(self)
 
-    def _uses_no_iter(self, w_obj):
-        return self.lookup(w_obj, '__iter__') is None
+    def _str_uses_no_iter(self, w_obj):
+        from pypy.objspace.descroperation import bytes_getitem
+        return (self.lookup(w_obj, '__iter__') is None and
+                self.lookup(w_obj, '__getitem__') is bytes_getitem(self))
+
+    def _uni_uses_no_iter(self, w_obj):
+        from pypy.objspace.descroperation import unicode_getitem
+        return (self.lookup(w_obj, '__iter__') is None and
+                self.lookup(w_obj, '__getitem__') is unicode_getitem(self))
 
     def sliceindices(self, w_slice, w_length):
         if isinstance(w_slice, W_SliceObject):
             a, b, c = w_slice.indices3(self, self.int_w(w_length))
             return (a, b, c)
-        w_indices = self.getattr(w_slice, self.wrap('indices'))
+        w_indices = self.getattr(w_slice, self.newtext('indices'))
         w_tup = self.call_function(w_indices, w_length)
         l_w = self.unpackiterable(w_tup)
         if not len(l_w) == 3:
@@ -536,7 +576,7 @@ class StdObjSpace(ObjSpace):
 
         # fast path: XXX this is duplicating most of the logic
         # from the default __getattribute__ and the getattr() method...
-        name = self.str_w(w_name)
+        name = self.text_w(w_name)
         w_descr = w_type.lookup(name)
         e = None
         if w_descr is not None:
@@ -604,7 +644,7 @@ class StdObjSpace(ObjSpace):
                 not w_obj.user_overridden_class):
             w_obj.setitem_str(key, w_value)
         else:
-            self.setitem(w_obj, self.wrap(key), w_value)
+            self.setitem(w_obj, self.newtext(key), w_value)
 
     def getindex_w(self, w_obj, w_exception, objdescr=None):
         if type(w_obj) is W_IntObject:
@@ -615,6 +655,10 @@ class StdObjSpace(ObjSpace):
         from pypy.objspace.std.unicodeobject import unicode_from_object
         return unicode_from_object(self, w_obj)
 
+    def encode_unicode_object(self, w_unicode, encoding, errors):
+        from pypy.objspace.std.unicodeobject import encode_object
+        return encode_object(self, w_unicode, encoding, errors)
+
     def call_method(self, w_obj, methname, *arg_w):
         return callmethod.call_method_opt(self, w_obj, methname, *arg_w)
 
@@ -644,4 +688,4 @@ class StdObjSpace(ObjSpace):
     @specialize.arg(2, 3)
     def is_overloaded(self, w_obj, tp, method):
         return (self.lookup(w_obj, method) is not
-                self.lookup_in_type_where(tp, method)[1])
+                self.lookup_in_type(tp, method))
