@@ -521,6 +521,10 @@ class LLFrame(object):
         if not x:
             raise LLAssertFailure(msg)
 
+    def op_debug_assert_not_none(self, x):
+        if not x:
+            raise LLAssertFailure("ll_assert_not_none() failed")
+
     def op_debug_fatalerror(self, ll_msg, ll_exc=None):
         msg = ''.join(ll_msg.chars)
         if ll_exc is None:
@@ -562,6 +566,9 @@ class LLFrame(object):
     def op_jit_conditional_call(self, *args):
         raise NotImplementedError("should not be called while not jitted")
 
+    def op_jit_conditional_call_value(self, *args):
+        raise NotImplementedError("should not be called while not jitted")
+
     def op_get_exception_addr(self, *args):
         raise NotImplementedError
 
@@ -956,6 +963,12 @@ class LLFrame(object):
     def op_gc_rawrefcount_create_link_pypy(self, *args):
         raise NotImplementedError("gc_rawrefcount_create_link_pypy")
 
+    def op_gc_rawrefcount_mark_deallocating(self, *args):
+        raise NotImplementedError("gc_rawrefcount_mark_deallocating")
+
+    def op_gc_rawrefcount_next_dead(self, *args):
+        raise NotImplementedError("gc_rawrefcount_next_dead")
+
     def op_do_malloc_fixedsize(self):
         raise NotImplementedError("do_malloc_fixedsize")
     def op_do_malloc_fixedsize_clear(self):
@@ -991,11 +1004,14 @@ class LLFrame(object):
     # __________________________________________________________
     # operations on addresses
 
-    def op_raw_malloc(self, size):
+    def op_raw_malloc(self, size, zero):
+        assert lltype.typeOf(size) == lltype.Signed
+        return llmemory.raw_malloc(size, zero=zero)
+
+    def op_boehm_malloc(self, size):
         assert lltype.typeOf(size) == lltype.Signed
         return llmemory.raw_malloc(size)
-
-    op_boehm_malloc = op_boehm_malloc_atomic = op_raw_malloc
+    op_boehm_malloc_atomic = op_boehm_malloc
 
     def op_boehm_register_finalizer(self, p, finalizer):
         pass
@@ -1063,9 +1079,6 @@ class LLFrame(object):
             assert offset.TYPE == ARGTYPE
             getattr(addr, str(ARGTYPE).lower())[offset.repeat] = value
 
-    def op_stack_malloc(self, size): # mmh
-        raise NotImplementedError("backend only")
-
     def op_track_alloc_start(self, addr):
         # we don't do tracking at this level
         checkadr(addr)
@@ -1073,6 +1086,65 @@ class LLFrame(object):
     def op_track_alloc_stop(self, addr):
         checkadr(addr)
 
+    def op_gc_enter_roots_frame(self, gcdata, numcolors):
+        """Fetch from the gcdata the current root_stack_top; bump it
+        by 'numcolors'; and assert that the new area is fully
+        uninitialized so far.
+        """
+        assert not hasattr(self, '_inside_roots_frame')
+        p = gcdata.inst_root_stack_top.ptr
+        q = lltype.direct_ptradd(p, numcolors)
+        self._inside_roots_frame = (p, q, numcolors, gcdata)
+        gcdata.inst_root_stack_top = llmemory.cast_ptr_to_adr(q)
+        #
+        array = p._obj._parentstructure()
+        index = p._obj._parent_index
+        for i in range(index, index + numcolors):
+            assert isinstance(array.getitem(i), lltype._uninitialized)
+
+    def op_gc_leave_roots_frame(self):
+        """Cancel gc_enter_roots_frame() by removing the frame from
+        the root_stack_top.  Writes uninitialized entries in its old place.
+        """
+        (p, q, numcolors, gcdata) = self._inside_roots_frame
+        assert gcdata.inst_root_stack_top.ptr == q
+        gcdata.inst_root_stack_top = llmemory.cast_ptr_to_adr(p)
+        del self._inside_roots_frame
+        #
+        array = p._obj._parentstructure()
+        index = p._obj._parent_index
+        for i in range(index, index + numcolors):
+            array.setitem(i, lltype._uninitialized(llmemory.Address))
+
+    def op_gc_save_root(self, num, value):
+        """Save one value (int or ptr) into the frame."""
+        (p, q, numcolors, gcdata) = self._inside_roots_frame
+        assert 0 <= num < numcolors
+        if isinstance(value, int):
+            assert value & 1    # must be odd
+            v = llmemory.cast_int_to_adr(value)
+        else:
+            v = llmemory.cast_ptr_to_adr(value)
+        llmemory.cast_ptr_to_adr(p).address[num] = v
+
+    def op_gc_restore_root(self, c_num, v_value):
+        """Restore one value from the frame."""
+        num = c_num.value
+        (p, q, numcolors, gcdata) = self._inside_roots_frame
+        assert 0 <= num < numcolors
+        assert isinstance(v_value.concretetype, lltype.Ptr)
+        assert v_value.concretetype.TO._gckind == 'gc'
+        newvalue = llmemory.cast_ptr_to_adr(p).address[num]
+        newvalue = llmemory.cast_adr_to_ptr(newvalue, v_value.concretetype)
+        self.setvar(v_value, newvalue)
+    op_gc_restore_root.specialform = True
+
+    def op_gc_push_roots(self, *args):
+        raise NotImplementedError
+
+    def op_gc_pop_roots(self, *args):
+        raise NotImplementedError
+
     # ____________________________________________________________
     # Overflow-detecting variants
 
@@ -1125,6 +1197,9 @@ class LLFrame(object):
         exc_data.exc_value = lltype.typeOf(evalue)._defl()
         return bool(etype)
 
+    def op_gc_move_out_of_nursery(self, obj):
+        raise NotImplementedError("gc_move_out_of_nursery")
+
 
 class Tracer(object):
     Counter = 0
